/***********************************************************
 * $Id: $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 08.04.2015
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.clazzes.util.http.osgi.DefaultHttpContext;
import org.clazzes.util.http.sec.HttpCheckLoginHelper;
import org.clazzes.util.http.sec.HttpLoginService;
import org.osgi.service.http.HttpContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/** Servlet for downloading a HttpResource (e.g. a file) given some path. Operates using the following steps.
 * 
 *  Step (1), determine the path:
 *  - Instance variable pathGetter is set: If the request parameter jobId is set, pass its value, or otherwise
 *        the result of calling request.getPathInfo() to the pathGetter.  Proceed with the path it returns.
 *  - Otherwise: Proceed with the path request.getPathInfo() returns.
 *  
 *  In both cases, the path is still one that uses "/" as separator character.
 * 
 *  Step (2), determine the ContentType based on the suffix after the last dot "." in the path.
 *  
 *  Step (3), determine the Content-Disposition.  The filename used for that (i.e. the filename presented to the
 *    user) is (a) the value of the request parameter "fileName" if set, or otherwise (b) the suffix after the last "/"
 *    of the path.
 *  
 *  Step (4), get the desired resource using the configured HttpResourceResolver.
 *  
 *  Usecases when a pathGetter needs to be used are e.g., if the file is the result of some job, i.e., needs to be queried
 *  by job id first, or generally if the path passed to the servlet does not contain the whole file name.
 *  
 *  In both cases, determining the ContentType and Content-Disposition would not be possible if the HttpResourceResolver would
 *  do all the job, as the latter just returns a HttpResource given a pathInfo.  I.e. if the path is transformed inside the 
 *  HttpResourceResolver, we don´t have any chance to inspect it in order to derive ContentType and Content-Disposition based
 *  on it.
 */
public class DownloadServlet extends HttpServlet {
    
    private static final long serialVersionUID = 4478910251103717663L;

    private static final Logger log = LoggerFactory.getLogger(DownloadServlet.class);
    
    private static final DateFormat RFC822_DATE_FORMAT = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss Z", Locale.US);    
    
    private Integer bufferSize;
    public void setBufferSize(Integer bufferSize) {
        this.bufferSize = bufferSize;
    }    
    
    private HttpResourceResolver resourceResolver;
    public void setResourceResolver(HttpResourceResolver resourceResolver) {
        this.resourceResolver = resourceResolver;
    }
    
    private HttpLoginService httpLoginService;
    public void setHttpLoginService(HttpLoginService httpLoginService) {
        this.httpLoginService = httpLoginService;
    }    
    
    private PathGetter pathGetter;
    public void setPathGetter(PathGetter pathGetter) {
        this.pathGetter = pathGetter;
    }
    
    private int maxAgeSeconds;
    public void setMaxAgeSeconds(int maxAgeSeconds) {
        this.maxAgeSeconds = maxAgeSeconds;
    } 
    
    public DownloadServlet() {
        this.maxAgeSeconds = 600;
    }
    
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
       
        if (this.httpLoginService != null && HttpCheckLoginHelper.checkLogin(this.httpLoginService, req, resp) == null) {
            log.warn("Refusing download without login");
            
            throw new SecurityException("Access denied.");
        }
        
        HttpContext httpContext = DefaultHttpContext.getInstance();
        
        String pathInfo;
        String jobId = req.getParameter("jobId");
        if (this.pathGetter != null) {
            // Determine path using pathGetter.
            
            String pathGetterParam = (jobId != null ? jobId : req.getPathInfo());
            pathInfo = this.pathGetter.getPath(pathGetterParam);
            
            if (log.isDebugEnabled()) {
                log.debug("Passed [" + pathGetterParam + "] to pathGetter and received pathInfo = [" + pathInfo + "].");
            }
            if (pathInfo == null) {
                throw new IllegalArgumentException("No pathInfo available for pathGetterParam [" + jobId + "]");
            }            
        } else {
            // Directly use req.getPathInfo()
            
            if (jobId != null) {
                throw new IllegalArgumentException("If no pathGetter is set, specifying the jobId is senseless and thus forbidden.  Did you configure the DownloadServlet correctly?");
            }
            
            pathInfo = req.getPathInfo();
            
            if (pathInfo.startsWith("/")) {
                pathInfo = pathInfo.substring(1);
            }            
            
            if (log.isDebugEnabled()) {
                log.debug("Received pathInfo = [" + pathInfo + "] from req.getPathInfo");
            }
        }
                
        String fileName = pathInfo;

        // Determine ContentType based on the suffix after the last dot (if a dot exists)
        int lastDotPosition = fileName.lastIndexOf('.');
        if (lastDotPosition != -1) {
            String suffix = fileName.substring(lastDotPosition + 1);

            String contentType = httpContext.getMimeType(suffix);
            resp.setContentType(contentType);                
            
            if (log.isDebugEnabled()) {
                log.debug("Set ContentType [" + contentType + "] based on path suffix [" + suffix + "]");
            }
        } else {
            if (log.isDebugEnabled()) {
                log.debug("Set no ContentType as the path contains no dot, i.e. no suffix could be determined.");
            }
        }
        
        // Determine Content-Disposition, i.e. the filename presented to the user
        String attachmentHeaderFileName = null;
        String fileNameParam = req.getParameter("fileName");
        if (fileNameParam != null) {
            attachmentHeaderFileName = fileNameParam;             // Case: Explicitely given
            
            if (log.isDebugEnabled()) {
                log.debug("Using attachmentHeaderFileName = [" + attachmentHeaderFileName + "] since this value is given as fileName parameter.");
            }
        } else {
            // Remove last separator, e.g. transfer foo/bar/ into foo/bar for the sake of this check
            String fileNameToCheck = fileName.endsWith("/") ? fileName.substring(0, fileName.length() - 1) : fileName;
            if (fileNameToCheck.contains("/")) { 
                // Case foo/bar
                attachmentHeaderFileName = fileNameToCheck.substring(fileNameToCheck.lastIndexOf("/") + 1);
            } else {
                // Case foo
                attachmentHeaderFileName = fileName;
            }
            
            if (log.isDebugEnabled()) {
                log.debug("Using attachmentHeaderFileName = [" + attachmentHeaderFileName + "] since this is the last token of the path."); 
            }            
        }

        String disposition = UrlHelper.getCompatibleContentDispositionAttachmentHeader(attachmentHeaderFileName);
        resp.setHeader("Content-Disposition", disposition);
        
        Date now = new Date();
        
        if (this.maxAgeSeconds == 0) {
            ResponseHelper.setNoCacheHeaders(resp);
        } else if (this.maxAgeSeconds > 0) {
            resp.setHeader("Expires", RFC822_DATE_FORMAT.format(new Date(now.getTime() + 1000L * this.maxAgeSeconds)));
            resp.setHeader("Cache-Control", "private, max-age=" + this.maxAgeSeconds);
        }
        
        HttpResource httpResource = this.resourceResolver.resolveResource(pathInfo, this.bufferSize != null ? this.bufferSize : 16384);
        
        try {
            httpResource.writeToStream(resp.getOutputStream());
        } finally {
            httpResource.close();
        }

        resp.getOutputStream().close();        
    }
    
    public String getServletInfo() {
        return "DownloadServlet";
    }
}
