/***********************************************************
 * $Id: InputStreamHttpResource.java 945 2014-03-21 08:17:26Z util $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 17.02.2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.MessageDigest;
import java.util.Date;

/**
 * A resource to be delivered, which is represented by an ordinary
 * input stream.
 */
public class InputStreamHttpResource implements HttpResource {

    private final InputStream inputStream;
    private final int bufferSize;
    
    /**
     * construct a resource from a given input stream and a given buffer
     * size for copying the stream to an output stream.
     * 
     * @param inputStream The stream representing the resource.
     * @param bufferSize The buffer size.
     */
    public InputStreamHttpResource(InputStream inputStream, int bufferSize) {
        super();
        this.inputStream = inputStream;
        this.bufferSize = bufferSize;
    }

    /**
     * construct a resource from a given input stream and a default buffer
     * size of 16384 for copying the stream to an output stream.
     * 
     * @param inputStream The stream representing the resource.
     */
    public InputStreamHttpResource(InputStream inputStream) {
        super();
        this.inputStream = inputStream;
        this.bufferSize = 16384;
    }

    @Override
    public void writeToStream(OutputStream os) throws IOException {
        
        byte[] buf = new byte[this.bufferSize];
        int n;
        
        while((n=this.inputStream.read(buf)) > 0) {
            os.write(buf, 0,n);
        }
    }

    @Override
    public long makeDigest(MessageDigest md) throws IOException {
       
        long l = 0;
        
        byte[] buf = new byte[this.bufferSize];
        int n;
        
        while((n=this.inputStream.read(buf)) > 0) {
            md.update(buf,0,n);
            l += n;
        }
        
        return l;
    }

    @Override
    public void readFromStream(InputStream is) throws IOException {
        
        throw new UnsupportedOperationException();
    }

    @Override
    public Date getLastModified() {
        
        return null;
    }

    @Override
    public boolean isUpdateable() {
        
        return false;
    }

    @Override
    public boolean isReusable() {
        
        return false;
    }

    @Override
    public void close() throws IOException {
        
        if (this.inputStream != null) {
            this.inputStream.close();
        }
    }

    /**
     * @return THe input stream representing this resource.
     */
    public InputStream getInputStream() {
        return this.inputStream;
    }

    /**
     * @return The buffer size for copying this resource to an output stream.
     */
    public int getBufferSize() {
        return this.bufferSize;
    }
    
    

}
