/***********************************************************
 * $Id: RequestHelper.java 1145 2016-02-26 20:08:05Z util $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 15.07.2012
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http;

import java.io.UnsupportedEncodingException;
import java.net.PasswordAuthentication;
import java.util.Enumeration;
import java.util.StringTokenizer;

import javax.servlet.Servlet;
import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.codec.binary.Base64;

/**
 * Static helper function for extracting special information from
 * requests.
 */
public abstract class RequestHelper {

	/**
	 * The attribute is set by delegating servlets, if the resolved
	 * resource name differs from
	 * {@link HttpServletRequest#getPathInfo()} and a delegate
	 * {@link Servlet} is called, which should be informed about a
	 * possible path resolution of the delegator.
	 */
	private static final String RESOLVED_PATH_ATTR = "org.clazzes.util.http::resolvedPath";

	/**
	 * @param req The request to inform about path resolution.
	 * @param resolvedPath The resolved server-side path to be propagted to a delegate
	 *                     servlet.
	 */
	public static void setResolvedPath(HttpServletRequest req, String resolvedPath) {
		req.setAttribute(RESOLVED_PATH_ATTR,resolvedPath);
	}
	
	/**
	 * The the request URL without host, port and protocol.
	 * 
	 * @param req The servlet request to get the URI from.
	 * @return The result of {@link HttpServletRequest#getRequestURI()} with
	 *         the value of {@link HttpServletRequest#getQueryString()}
	 *         appended alongside with a question mark,
	 *         if the query string is not <code>null</code>.
	 */
	public static String getRequestUrl(HttpServletRequest req) {
	    
	    if (req.getQueryString() == null) {
	        return req.getRequestURI();
	    }
	    else {
	        return req.getRequestURI() + "?" + req.getQueryString();
	    }
	}

	/**
	 * Return a string describing a servlet request for logging purposes.
	 * 
	 * @param req The servlet request to describe.
	 * @return A string ctaining the request URL for HTTP requests
	 *         or a string containing the request protocol for
	 *         other requests.
	 */
	public static String describeRequest(ServletRequest req) {
	
	    if (req instanceof HttpServletRequest) {
	        return "HTTP request to ["+getRequestUrl((HttpServletRequest)req)+"]";
	    }
	    else {
	        return "Request with protocol ["+req.getProtocol()+"]";
	    }
	}
	
	/**
	 * Get the server-side path which may have been resolved by a delegating
	 * servlet.
	 * 
	 * @param req The request to query.
	 * @return The resolved path stored in the {@link #RESOLVED_PATH_ATTR} attribute
	 *         or The result of {@link HttpServletRequest#getPathInfo()}, if
	 *         no such attribute is present.
	 */
	public static String getResolvedPath(HttpServletRequest req) {
		
		Object rp = req.getAttribute(RESOLVED_PATH_ATTR);
		
		if (rp == null) {
			String ret = req.getPathInfo();
			
			if (ret == null) {
			    ret = "";
			}
			return ret;
		}
		else {
			// don't cast to string, calling String.toString() is faster.
			return rp.toString();
		}
	}
	
	/**
	 * <p>Get the remote IP address of the client taking the
	 * <code>X-Forwarded-for</code> header into account.</p>
	 * 
	 * <p>If no <code>X-Forwarded-for</code> header is present,
	 * the value of {@link HttpServletRequest#getRemoteAddr()}
	 * is returned.</p>
	 * 
	 * <p>If a <code>X-Forwarded-for</code> header is present,
	 * the leftmost value in the comma-separated list of
	 * <code>X-Forwarded-for</code>-addresses is returned.
	 * The rationale for returning the leftmost address is, that
	 * this is the original address of the client.</p>
	 * 
	 * @param request A HTTP servlet request.
	 * @return The remote IP address of the client.
	 */
	public static String getRealRemoteIP(final HttpServletRequest request) {
		@SuppressWarnings("unchecked")
		final Enumeration<String> headers = request
				.getHeaders("X-Forwarded-for");
		if (headers != null) {
			while (headers.hasMoreElements()) {
				final String[] ips = headers.nextElement().split(",");
				for (int i = 0; i < ips.length; i++) {
					final String realRemoteAddr = ips[i].trim();
					if (!"unknown".equals(realRemoteAddr)
							&& !realRemoteAddr.isEmpty()) {
						return realRemoteAddr;
					}
				}
			}
		}
		return request.getRemoteAddr();
	}

	/**
	 * <p>Get the value of the servlet parameter <code>key</code> and parse it
	 * using {@link Double#parseDouble(String)} if a value for this servlet parameter
	 * has been found.</p>
	 * 
	 * <p><b>Warning</b>:
	 * For POST requests with a Content-Type <code>application/x-www-form-urlencoded</code> 
	 * this method triggers a parsing of the all posted query parameters.
	 * To avoid this, use {@link UrlHelper#parseQuery(String)} together with
	 * {@link UrlHelper#getDoubleParameter(java.util.Map, String)}.</p>
	 * 
	 * @param req The HTTP servlet request to query.
	 * @param key The key of the double parameter to parse.
	 * @return The double value of the request parameter with the given key or
	 *         <code>null</code> if no request parameter with name <code>key</code>
	 *         is given. 
	 * @throws NumberFormatException If the value of the servlet parameter is
	 *         not a double value as per {@link Double#parseDouble(String)}.
	 */
	public static Double getDoubleParameter(HttpServletRequest req, String key) throws NumberFormatException {

		 String s = req.getParameter(key);
		 
		 if (s == null) {
			 return null;
		 } else {
			 
			 try {
				 return Double.valueOf(s);
			 }
			 catch(NumberFormatException e) {
				 throw new NumberFormatException("The value ["+s+"] of servlet parameter ["+key+"] is not a valid double precision number.");
			 }
		 }
	 }

	/**
	 * <p>Get the value of the servlet parameter <code>key</code> and parse it
	 * using {@link Long#parseLong(String)} if a value for this servlet parameter
	 * has been found.</p>
	 * 
   * <p><b>Warning</b>:
   * For POST requests with a Content-Type <code>application/x-www-form-urlencoded</code> 
   * this method triggers a parsing of the all posted query parameters.
   * To avoid this, use {@link UrlHelper#parseQuery(String)} together with
	 * {@link UrlHelper#getLongParameter(java.util.Map, String)}.</p>
	 * 
	 * @param req The HTTP servlet request to query.
	 * @param key The key of the long parameter to parse.
	 * @return The long value of the request parameter with the given key or
	 *         <code>null</code> if no request parameter with name <code>key</code>
	 *         is given. 
	 * @throws NumberFormatException If the value of the servlet parameter is
	 *         not a long value as per {@link Long#parseLong(String)}.
	 */
	public static Long getLongParameter(HttpServletRequest req, String key) throws NumberFormatException {

		 String s = req.getParameter(key);
		 
		 if (s == null) {
			 return null;
		 } else {
			 try {
				 return Long.valueOf(s);
			 }
			 catch(NumberFormatException e) {
				 throw new NumberFormatException("The value ["+s+"] of servlet parameter ["+key+"] is not a valid long integer number.");
			 }
		 }
	 }

	/**
	 * <p>Get the value of the servlet parameter <code>key</code> and parse it
	 * using {@link Integer#parseInt(String)} if a value for this servlet parameter
	 * has been found.</p>
	 * 
   * <p><b>Warning</b>:
   * For POST requests with a Content-Type <code>application/x-www-form-urlencoded</code> 
   * this method triggers a parsing of the all posted query parameters.
   * To avoid this, use {@link UrlHelper#parseQuery(String)} together with
	 * {@link UrlHelper#getIntegerParameter(java.util.Map, String)}.</p>
	 * 
	 * @param req The HTTP servlet request to query.
	 * @param key The key of the integer parameter to parse.
	 * @return The integer value of the request parameter with the given key or
	 *         <code>null</code> if no request parameter with name <code>key</code>
	 *         is given. 
	 * @throws NumberFormatException If the value of the servlet parameter is
	 *         not an integer value as per {@link Integer#parseInt(String)}.
	 */
	public static Integer getIntegerParameter(HttpServletRequest req, String key) throws NumberFormatException {

		 String s = req.getParameter(key);
		 
		 if (s == null) {
			 return null;
		 } else {
			 try {
				 return Integer.valueOf(s);
			 }
			 catch(NumberFormatException e) {
				 throw new NumberFormatException("The value ["+s+"] of servlet parameter ["+key+"] is not a valid integer number.");
			 }
		 }
	 }

	/**
	 * <p>Get the value of the servlet parameter <code>key</code> and parse it
	 * using {@link Boolean#parseBoolean(String)} if a value for this servlet parameter
	 * has been found.</p>
	 * 
	 * <p><b>Warning</b>:
	 * For POST requests with a Content-Type <code>application/x-www-form-urlencoded</code> 
	 * this method triggers a parsing of the all posted query parameters.
	 * To avoid this, use {@link UrlHelper#parseQuery(String)} together with
	 * {@link UrlHelper#getBooleanParameter(java.util.Map, String)}.</p>
	 * 
	 * @param req The HTTP servlet request to query.
	 * @param key The key of the boolean parameter to parse.
	 * @return The boolean value of the request parameter with the given key or
	 *         <code>null</code> if no request parameter with name <code>key</code>
	 *         is given. 
	 * @throws NumberFormatException If the value of the servlet parameter is
	 *         not an integer value as per {@link Boolean#parseBoolean(String)}.
	 */
	public static Boolean getBooleanParameter(HttpServletRequest req, String key) {

		 String s = req.getParameter(key);
		 
		 if (s == null) {
			 return null;
		 } else {
			 return Boolean.valueOf(s);
		 }
	 }
	
	/**
	 * Parse the <code>Authorization</code> request header and extract
	 * HTTP basic authentication credentials.
	 * 
	 * @param req The request to inspect.
	 * @return The user and password contained in <code>Authorization: Basic</code>
	 *         or <code>null</code>, if no <code>Authorization</code>
	 *         header is given or the header referred to another authorization
	 *         scheme.
	 * @throws SecurityException When the <code>Authorization</code> header
	 *         is malformed.
	 */
	public static final PasswordAuthentication parseBasicAuthenticationHeader(HttpServletRequest req) {
	       
        String authHeader = req.getHeader("Authorization");
        
        if (authHeader != null) {
            StringTokenizer st = new StringTokenizer(authHeader);
            if (st.hasMoreTokens()) {
                String basic = st.nextToken();

                // We only handle HTTP Basic authentication
                if (basic.equalsIgnoreCase("Basic")) {
                    
                    String credentials = st.nextToken();

                    // Use OSGi-compatible commons-codec for base64 decoding.
                    String userPass;
                    try {
                        userPass = new String(Base64.decodeBase64(credentials),"UTF-8");
                    } catch (UnsupportedEncodingException e) {
                        throw new SecurityException("Unable to decode basic auth header, UTF-8 encoding unknown",e);
                    }
                    
                    // The decoded string is in the form
                    // "userID:password"
                    
                    int p = userPass.indexOf(":");
                    
                    if (p != -1) {
                        
                        String userID = userPass.substring(0, p);
                        String password = userPass.substring(p+1);                      
                        
                        return new PasswordAuthentication(userID,password.toCharArray());
                        
                    } else {
                        throw new SecurityException("Failed to fetch information from Authorization header.");
                    }
                    
                } else {
                    //
                    // Prefer to return null, if the header belongs to another
                    // authentication scheme.
                    //
                    // throw new SecurityException("Failed to recognize authentication type.");
                    return null;
                }
            } else {
                throw new SecurityException("Failed to fetch authHeader tokens.");                
            }
        } else {
            return null;
        }
	}
	   
}
