/***********************************************************
 * $Id: ResponseHelper.java 1089 2015-09-12 21:17:24Z util $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 19.08.2011
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http;

import java.io.IOException;

import javax.servlet.http.HttpServletResponse;

/**
 * Static helper functions for frequently needed additions to HTTP responses.
 */
public abstract class ResponseHelper {

    /**
     * The system property determining the default value for
     * the {@link #isHideStackTraces()} property.
     */
    public static String HIDE_STACK_TRACES_PROPERTY = "org.clazzes.util.http.hideStackTraces";

    private static final boolean hideStackTraces;
    
    static {
        hideStackTraces =
                Boolean.parseBoolean(System.getProperty(HIDE_STACK_TRACES_PROPERTY,"true"));
    }
    
    /**
     * @param hideStackTraces Whether the registered servlets will be covered
     *         by ExceptionShadowingServlet
     *         in order to hide stack traces from clients for security
     *         reasons.
     *         The value is <code>true</code> or the value
     *         of the {@value ResponseHelper#HIDE_STACK_TRACES_PROPERTY} system
     *         property, if present.
     */
    public static boolean isHideStackTraces() {
        return hideStackTraces;
    }
    
    /**
	 * Set the following HTTP headers fields in order to mark the servlet's
	 * output as purely dynamic content not to be cached by any browser.
	 * 
	 * <pre>
	 * Cache-Control: no-cache
	 * Pragma: no-cache
	 * Expires: 0
	 * </pre>
	 * 
	 * @param resp The servlet response to be marked as dynamic content.
	 */
	public static final void setNoCacheHeaders(HttpServletResponse resp) {
		resp.setHeader("Cache-Control","no-cache");
		resp.setHeader("Pragma","no-cache");
		resp.setHeader("Expires","0");
	}
	
	/**
	 * Send an <code>WWW-Authenticate: Basic realm="&lt;realm&gt;"</code> header and
	 * and authorization required body to the servlet response in order
	 * to tell the client to send a basic authorization header.
	 * 
	 * @param resp The response to send the authorization request on.
	 * @param realm The realm matching the regexp <code>^[a-zA-Z0-9_ -]*$</code>.
	 * @throws IOException Upon write errors.
	 */
	public static final void sendBasicAuthenticationRequired(HttpServletResponse resp, String realm) throws IOException {
	    
	    if (!realm.matches("^[a-zA-Z0-9_ -]*$")) {
	        throw new SecurityException("Realm contains other characters than [a-zA-Z0-9_ -].");
	    }
	    
	    String s = "Basic realm=\""+realm+"\"";
	    resp.setHeader("WWW-Authenticate", s);
	    ResponseHelper.setNoCacheHeaders(resp);
	    resp.setStatus(401);
	    resp.getOutputStream().println("<html><body>Authorization Required</body></html>");
	    resp.getOutputStream().flush();     

	}
}
