/***********************************************************
 * $Id: URLHttpResource.java 954 2014-03-23 16:44:30Z util $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 17.02.2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.security.MessageDigest;
import java.util.Date;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A resource to be delivered, which is represented by an URL.
 */
public class URLHttpResource implements HttpResource {

    private static final Logger log = LoggerFactory.getLogger(URLHttpResource.class);
    
    private final int bufferSize;
    private final URL url;
    
    private String writeReplaceRegex;
    private String writeReplacement;
    
    /**
     * construct a resource from a given input stream and a given buffer
     * size for copying the stream to an output stream.
     * 
     * @param url The URL representing the resource
     * @param bufferSize The buffer size.
     */
    public URLHttpResource(URL url, int bufferSize) {
        super();
        this.url = url;
        this.bufferSize = bufferSize;
    }

    /**
     * construct a resource from a given input stream and a default buffer
     * size of 16384 for copying the stream to an output stream.
     * 
     * @param inputStream The stream representing the resource.
     */
    public URLHttpResource(URL url) {
        super();
        this.url = url;
        this.bufferSize = 16384;
    }
    
    /**
     * Configure a file name mapping for writing.
     * 
     * @param regex The regular expression to replace.
     * @param repl The replacement.
     * @see String#replaceFirst(String, String)
     */
    public void setWritePlacement(String regex, String repl) {
        
        this.writeReplaceRegex = regex;
        this.writeReplacement = repl;
    }

    @Override
    public void writeToStream(OutputStream os) throws IOException {
        
        InputStream is = this.url.openConnection().getInputStream();
        
        try {
            byte[] buf = new byte[this.bufferSize];
            int n;
            
            while((n=is.read(buf)) > 0) {
                os.write(buf, 0,n);
            }
        }
        finally {
            is.close();
        }
    }

    @Override
    public long makeDigest(MessageDigest md) throws IOException {
       
        InputStream is = this.url.openConnection().getInputStream();
        
        try {
            long l = 0;
            
            byte[] buf = new byte[this.bufferSize];
            int n;
            
            while((n=is.read(buf)) > 0) {
                md.update(buf,0,n);
                l += n;
            }

            return l;
        }
        finally {
            is.close();
        }
    }

    @Override
    public void readFromStream(InputStream is) throws IOException {
        
        if (!this.isUpdateable()) {
            throw new UnsupportedOperationException();
        }
        
        String path = this.url.getFile();
        String path2 = null;
        
        if (this.writeReplacement != null && this.writeReplaceRegex != null) {
            
            path2 = path.replaceFirst(this.writeReplaceRegex,this.writeReplacement);
        }
        
        File file = new File(path);
        
        if (!file.exists()) {
            throw new IOException("File ["+file+"] may not be updated, beacuse it does not exist.");
        }
        
        OutputStream os = new FileOutputStream(file);
        OutputStream os2 = null;
        
        File file2 = null;
        
        if (path2 != null) {
            
            file2 = new File(path2);
            
            if (file2.exists()) {
                log.info("Updating replacement resource at path [{}].",path2);
                os2 = new FileOutputStream(path2);
            }
            else {
                log.warn("Not updating replacement resource at path [{}], beacuse it does not exist.",path2);
            }
        }
        
        try {
            byte[] buf = new byte[this.bufferSize];
            int n;
            
            while((n=is.read(buf)) > 0) {
                os.write(buf, 0,n);
                
                if (os2 != null) {
                    os2.write(buf, 0,n);
                }
            }
        }
        finally {
            
            try {
                os.close();    
            }
            catch (IOException e) {
                log.warn("Error closing output file ["+file+"]",e);
            }
            
            if (os2 != null) {
                try {
                    os2.close();
                }
                catch(IOException e) {
                    log.warn("Error closing replacement file ["+file2+"]",e);
                }
            }
        }
    }

    @Override
    public Date getLastModified() {
        
        // return a modification date for file URLs
        if (this.isUpdateable()) {
            
            File f = new File(this.url.getFile());
            
            if (f.exists()) {
                return new Date(f.lastModified());
            }
        }
        
        return null;
    }

    @Override
    public boolean isUpdateable() {
        
        return "file".equals(this.url.getProtocol());
    }

    @Override
    public boolean isReusable() {
        
        return false;
    }

    @Override
    public void close() throws IOException {
        
    }

    /**
     * @return The URL representing this resource.
     */
    public URL getUrl() {
        return this.url;
    }

    /**
     * @return The buffer size for copying this resource to an output stream.
     */
    public int getBufferSize() {
        return this.bufferSize;
    }
    
    

}
