/***********************************************************
 * $Id: UrlHelper.java 1026 2015-04-08 14:46:23Z util $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 19.08.2011
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http;

import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.nio.charset.IllegalCharsetNameException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Static helpers for manipulating URLs and URIs.
 */
public class UrlHelper {

	/**
	 * The name of the UTF-8 encoding for {@link Charset#forName(String)}.
	 */
	public static final String UTF_8 = "UTF-8";
	
	/**
     * Translates a string into <code>application/x-www-form-urlencoded</code>
     * format using UTF-8 encoding as stated in the <a href=
     * "http://www.w3.org/TR/html40/appendix/notes.html#non-ascii-chars">
     * World Wide Web Consortium Recommendation</a>.
     * <br><br>
     * In contrast to <code>URLEncoder.encode(s,"UTF-8")</code> this function
     * does not throw any checked exception.
     *
     * @param   s   <code>String</code> to be translated.
     * @return  the translated <code>String</code>.
     * @see URLEncoder#encode(java.lang.String, java.lang.String)
     */
	public static final String encodeUtf8(String s) {
		try {
			return URLEncoder.encode(s,UTF_8);
		} catch (UnsupportedEncodingException e) {
			throw new IllegalCharsetNameException(UTF_8);
		}
	}
	
    /**
     * Decodes a <code>application/x-www-form-urlencoded</code> string using
     * UTF-8 encoding as stated in the <a href=
     * "http://www.w3.org/TR/html40/appendix/notes.html#non-ascii-chars">
     * World Wide Web Consortium Recommendation</a>.
     * <br><br>
     * In contrast to <code>URLDecoder.decode(s,"UTF-8")</code> this function
     * does not throw any checked exception.
     *
     * @param s the <code>String</code> to decode.
     * @return the newly decoded <code>String</code>
     * @see URLDecoder#decode(String, String)
     */
	public static final String decodeUtf8(String s) {
		try {
			return URLDecoder.decode(s,UTF_8);
		} catch (UnsupportedEncodingException e) {
			throw new IllegalCharsetNameException(UTF_8);
		}
	}
	
	public static String getCompatibleContentDispositionAttachmentHeader(String fileName) {
        String encodedFileNameBase = UrlHelper.encodeUtf8(fileName);
        String encodedFileName = encodedFileNameBase.replace("+", "%20");
        String encodedFileNameTraditional = encodedFileNameBase.replace("+"," ");
        
        //
        // Encode according to the new schema defined in RFC2231/5987 using UTF-8
        //
        // http://greenbytes.de/tech/tc2231/#attfnboth
        //
        // Content-Disposition: attachment; filename="foo-ae.html"; filename*=UTF-8''foo-%c3%a4.html
        //        
        String disposition = "attachment; filename=\"" + encodedFileNameTraditional + "\"; filename*=UTF-8''" + encodedFileName;
        
        return disposition;
	}
	
	/**
	 * Append additional query parameters to the end of a given URL.
	 * 
	 * @param url The base URL, which my already contain other query parameters.
	 * @param key The name of the parameter.
	 * @param value The value, which will be appended using {@link #encodeUtf8(String)}.
	 * @return The base URL extended by an additional query parameter.
	 */
	public static final String appendQueryParameterToUrl(String url, String key, String value) {
		
		if (value == null) {
			
			if (url.contains("?")) {
				return url+"&"+encodeUtf8(key);
			} else {
				return url+"?"+encodeUtf8(key);
			}
		}
		else {
		
			if (url.contains("?")) {
				return url+"&"+encodeUtf8(key)+"="+encodeUtf8(value);
			} else {
				return url+"?"+encodeUtf8(key)+"="+encodeUtf8(value);
			}
		}
	}

	/**
	 * Append additional query parameters to the end of a given query string.
	 * 
	 * @param q The query part of an URL or URI as returned by {@link URL#getQuery()} or {@link URI#getQuery()}.
	 * @param key The name of the parameter.
	 * @param value The value, which will be appended using {@link #encodeUtf8(String)}.
	 * @return The query extended by an additional query parameter.
	 */
	public static final String appendQueryParameter(String q, String key, String value) {
		
		if (value == null) {
						
			if (q != null && q.length() > 0) {
				return q +"&"+encodeUtf8(key);
			} else {
				return encodeUtf8(key);
			}
		}
		else {
			if (q != null && q.length() > 0) {
				return q +"&"+encodeUtf8(key)+"="+encodeUtf8(value);
			} else {
				return encodeUtf8(key)+"="+encodeUtf8(value);
			}
		}
	}

	/**
	 * Parse a given query string in <code>application/x-www-form-urlencoded</code>
     * format using UTF-8 encoding.
	 * 
	 * @param q The query part of an URL or URI as returned by {@link URL#getQuery()}
	 *          or {@link URI#getQuery()}.
	 * @return A map of query parameters with their values. A parameter value
	 *         may be manifold, so the value is represented by a list of strings.
	 *         If <code>q==null</code>, the return value is also <code>null</code>.
	 * @see #getStringParameter(Map, String)
	 * @see #getDoubleParameter(Map, String)
	 * @see #getLongParameter(Map, String)
	 * @see #getIntegerParameter(Map, String)
	 * @see #getBooleanParameter(Map, String)
	 */
	public static final Map<String,List<String>> parseQuery(String q) {
		
		if (q==null) {
			return null;
		}
		
		final Map<String,List<String>> ret = new HashMap<String,List<String>>();
		
		UrlHelper.parseQuery(q,new UrlParameterConsumer() {
            
            @Override
            public boolean addParameter(String key, String value) {

                List<String> ov = ret.get(key);
                
                if (ov == null) {
                    ov = new ArrayList<String>();
                    ret.put(key,ov);
                }
                ov.add(value);
                
                return false;
           }
        });		
		
		return ret;
	}
	
	/**
     * Parse a given query string in <code>application/x-www-form-urlencoded</code>
     * format using UTF-8 encoding.
     * 
     * @param q The query part of an URL or URI as returned by {@link URL#getQuery()}
     *          or {@link URI#getQuery()}.
     * @param consumer A callback for reporting parameters as they appear. A parameter value
     *         may be manifold, so a parameter key may be reported multiple times.
     * @see #getStringParameter(String, String)
     * @see #getDoubleParameter(String, String)
     * @see #getLongParameter(String, String)
     * @see #getIntegerParameter(String, String)
     * @see #getBooleanParameter(String, String)
     */
    public static final void parseQuery(String q, UrlParameterConsumer consumer) {

        if (q==null) {
            return;
        }
        
        int pos = 0;
        
        do {
            int p1 = q.indexOf('=',pos);
            int p2;
            
            String key;
            String value;
            
            if (p1 < 0) {
                p2 = q.indexOf('&',pos);
                
                if (p2 < 0) {
                    key = decodeUtf8(q.substring(pos));
                    pos = q.length();
                } else {
                    key = decodeUtf8(q.substring(pos,p2));
                    pos = p2+1;
                }
                value = null;
            } else {
                p2 = q.indexOf('&',pos);

                if (p2 < 0) {
                    key = decodeUtf8(q.substring(pos,p1));
                    value = decodeUtf8(q.substring(p1+1));
                    pos = q.length();
                } else if (p2 < p1) {
                    key = decodeUtf8(q.substring(pos,p2));
                    value = null;
                    pos = p2+1;
                } else {
                    key = decodeUtf8(q.substring(pos,p1));
                    value = decodeUtf8(q.substring(p1+1,p2));
                    pos = p2+1;
                }
            }
            
            if (consumer.addParameter(key, value)) {
                break;
            }
            
        } while (pos < q.length());
    }
    
	/**
	 * Get the first string value given in the specified map of
	 * query parameters. 
	 * 
	 * @param parameters A map of query parameters as returned by {@link #parseQuery(String)}.
	 * @param key The name of the parameter to query.
	 * @return The first string value for the given parameter name or <code>null</code>,
	 *         if no such parameter is given.
	 */
	public static String getStringParameter(Map<String,List<String>> parameters, String key) {
		
		if (parameters == null) {
			return null;
		}
		else {
		
			List<String> values = parameters.get(key);
			
			if (values == null || values.size() <= 0) {
				return null;
			}
			else {
				return values.get(0);
			}
		}
	}

    /**
     * Get the first string value given in the specified map of
     * query parameters.
     * 
     * If you a parsing multiple keys, use the more performing combination
     * of {@link #parseQuery(String)} and {@link #getStringParameter(Map, String)}
     * instead.
     * 
     * @param q The query part of an URL or URI as returned by {@link URL#getQuery()}
     *          or {@link URI#getQuery()}.
     * @param key The name of the parameter to query.
     * @return The first string value for the given parameter name or <code>null</code>,
     *         if no such parameter is given.
     */
    public static String getStringParameter(String q, String key) {
        
        SingletonUrlParameterConsumer sc = new SingletonUrlParameterConsumer(key);
        
        UrlHelper.parseQuery(q,sc);
        
        return sc.getValue();
    }

    /**
	 * Get the value of the query parameter <code>key</code> and parse it
	 * using {@link Double#parseDouble(String)} if a value for this query parameter
	 * has been found.
	 * 
	 * @param parameters A map of query parameters as returned by {@link #parseQuery(String)}.
	 * @param key The key of the double parameter to parse.
	 * @return The double value of the query parameter with the given key or
	 *         <code>null</code> if no query parameter with name <code>key</code>
	 *         is given. 
	 * @throws NumberFormatException If the value of the query parameter is
	 *         not a double value as per {@link Double#parseDouble(String)}.
	 */
	public static Double getDoubleParameter(Map<String,List<String>> parameters, String key) throws NumberFormatException {
		
		String s = getStringParameter(parameters, key);
		
		if (s == null) {
			return null;
		} else {
			
			try {
				return Double.valueOf(s);
			}
			catch(NumberFormatException e) {
				throw new NumberFormatException("The value ["+s+"] of query parameter ["+key+"] is not a valid double precision number.");
			 }
		}
	}

    /**
     * Get the value of the query parameter <code>key</code> and parse it
     * using {@link Double#parseDouble(String)} if a value for this query parameter
     * has been found.
     * 
     * If you a parsing multiple keys, use the more performing combination
     * of {@link #parseQuery(String)} and {@link #getDoubleParameter(Map, String)}
     * instead.
     * 
     * @param q The query part of an URL or URI as returned by {@link URL#getQuery()}
     *          or {@link URI#getQuery()}.
     * @param key The key of the double parameter to parse.
     * @return The double value of the query parameter with the given key or
     *         <code>null</code> if no query parameter with name <code>key</code>
     *         is given. 
     * @throws NumberFormatException If the value of the query parameter is
     *         not a double value as per {@link Double#parseDouble(String)}.
     */
    public static Double getDoubleParameter(String q, String key) throws NumberFormatException {
        
        String s = getStringParameter(q, key);
        
        if (s == null) {
            return null;
        } else {
            
            try {
                return Double.valueOf(s);
            }
            catch(NumberFormatException e) {
                throw new NumberFormatException("The value ["+s+"] of query parameter ["+key+"] is not a valid double precision number.");
             }
        }
    }

    /**
	 * Get the value of the query parameter <code>key</code> and parse it
	 * using {@link Long#parseLong(String)} if a value for this query parameter
	 * has been found.
	 * 
	 * @param parameters A map of query parameters as returned by {@link #parseQuery(String)}.
	 * @param key The key of the long parameter to parse.
	 * @return The long value of the query parameter with the given key or
	 *         <code>null</code> if no query parameter with name <code>key</code>
	 *         is given. 
	 * @throws NumberFormatException If the value of the query parameter is
	 *         not a long value as per {@link Long#parseLong(String)}.
	 */
	public static Long getLongParameter(Map<String,List<String>> parameters, String key) throws NumberFormatException {
		
		String s = getStringParameter(parameters, key);
		
		if (s == null) {
			return null;
		} else {
			
			try {
				return Long.valueOf(s);
			}
			catch(NumberFormatException e) {
				 throw new NumberFormatException("The value ["+s+"] of query parameter ["+key+"] is not a valid long integer number.");
			}
		}
	}

    /**
     * Get the value of the query parameter <code>key</code> and parse it
     * using {@link Long#parseLong(String)} if a value for this query parameter
     * has been found.
     * 
     * If you a parsing multiple keys, use the more performing combination
     * of {@link #parseQuery(String)} and {@link #getLongParameter(Map, String)}
     * instead.
     * 
     * @param q The query part of an URL or URI as returned by {@link URL#getQuery()}
     *          or {@link URI#getQuery()}.
     * @param key The key of the long parameter to parse.
     * @return The long value of the query parameter with the given key or
     *         <code>null</code> if no query parameter with name <code>key</code>
     *         is given. 
     * @throws NumberFormatException If the value of the query parameter is
     *         not a long value as per {@link Long#parseLong(String)}.
     */
    public static Long getLongParameter(String q, String key) throws NumberFormatException {
        
        String s = getStringParameter(q, key);
        
        if (s == null) {
            return null;
        } else {
            
            try {
                return Long.valueOf(s);
            }
            catch(NumberFormatException e) {
                 throw new NumberFormatException("The value ["+s+"] of query parameter ["+key+"] is not a valid long integer number.");
            }
        }
    }

    /**
	 * Get the value of the query parameter <code>key</code> and parse it
	 * using {@link Integer#parseInt(String)} if a value for this query parameter
	 * has been found.
	 * 
	 * @param parameters A map of query parameters as returned by {@link #parseQuery(String)}.
	 * @param key The key of the long parameter to parse.
	 * @return The integer value of the query parameter with the given key or
	 *         <code>null</code> if no query parameter with name <code>key</code>
	 *         is given. 
	 * @throws NumberFormatException If the value of the query parameter is
	 *         not an integer value as per {@link Integer#parseInt(String)}.
	 */
	public static Integer getIntegerParameter(Map<String,List<String>> parameters, String key) throws NumberFormatException {
		
		String s = getStringParameter(parameters, key);
		
		if (s == null) {
			return null;
		} else {
			
			try {
				return Integer.valueOf(s);
			}
			catch(NumberFormatException e) {
				throw new NumberFormatException("The value ["+s+"] of query parameter ["+key+"] is not a valid integer number.");
			}
		}
	}

    /**
     * Get the value of the query parameter <code>key</code> and parse it
     * using {@link Integer#parseInt(String)} if a value for this query parameter
     * has been found.
     * 
     * If you a parsing multiple keys, use the more performing combination
     * of {@link #parseQuery(String)} and {@link #getIntegerParameter(Map, String)}
     * instead.
     * 
     * @param q The query part of an URL or URI as returned by {@link URL#getQuery()}
     *          or {@link URI#getQuery()}.
     * @param key The key of the long parameter to parse.
     * @return The integer value of the query parameter with the given key or
     *         <code>null</code> if no query parameter with name <code>key</code>
     *         is given. 
     * @throws NumberFormatException If the value of the query parameter is
     *         not an integer value as per {@link Integer#parseInt(String)}.
     */
    public static Integer getIntegerParameter(String q, String key) throws NumberFormatException {
        
        String s = getStringParameter(q, key);
        
        if (s == null) {
            return null;
        } else {
            
            try {
                return Integer.valueOf(s);
            }
            catch(NumberFormatException e) {
                throw new NumberFormatException("The value ["+s+"] of query parameter ["+key+"] is not a valid integer number.");
            }
        }
    }

    /**
	 * Get the value of the query parameter <code>key</code> and parse it
	 * using {@link Boolean#parseBoolean(String)} if a value for this query parameter
	 * has been found.
	 * 
	 * @param parameters A map of query parameters as returned by {@link #parseQuery(String)}.
	 * @param key The key of the long parameter to parse.
	 * @return The boolean value of the query parameter with the given key or
	 *         <code>null</code> if no query parameter with name <code>key</code>
	 *         is given. 
	 */
	public static Boolean getBooleanParameter(Map<String,List<String>> parameters, String key) {
		
		String s = getStringParameter(parameters, key);
		
		if (s == null) {
			return null;
		} else {
			
			return Boolean.valueOf(s);
		}
	}

    /**
     * Get the value of the query parameter <code>key</code> and parse it
     * using {@link Boolean#parseBoolean(String)} if a value for this query parameter
     * has been found.
     * 
     * If you a parsing multiple keys, use the more performing combination
     * of {@link #parseQuery(String)} and {@link #getIntegerParameter(Map, String)}
     * instead.
     * 
     * @param q The query part of an URL or URI as returned by {@link URL#getQuery()}
     *          or {@link URI#getQuery()}.
     * @param key The key of the long parameter to parse.
     * @return The boolean value of the query parameter with the given key or
     *         <code>null</code> if no query parameter with name <code>key</code>
     *         is given. 
     */
    public static Boolean getBooleanParameter(String q, String key) {
        
        String s = getStringParameter(q, key);
        
        if (s == null) {
            return null;
        } else {
            
            return Boolean.valueOf(s);
        }
    }
}
