/***********************************************************
 * $Id: ExceptionShadowingServlet.java 1089 2015-09-12 21:17:24Z util $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 11.09.2015
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http.osgi;

import java.io.IOException;

import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.UnavailableException;

import org.clazzes.util.http.RequestHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A servlet, which hides stack traces from the the servlet container
 * and avoid the exposure of full stack traces to clients.
 */
public class ExceptionShadowingServlet implements Servlet {

    private static final Logger log = LoggerFactory.getLogger(ExceptionShadowingServlet.class); 
    
    private final Servlet delegate;
    
    /**
     * @param delegate The delegate servlet to shadow.
     */
    public ExceptionShadowingServlet(Servlet delegate) {
        super();
        this.delegate = delegate;
    }

    /* (non-Javadoc)
     * @see javax.servlet.Servlet#destroy()
     */
    @Override
    public void destroy() {
        
        this.delegate.destroy();
    }

    /* (non-Javadoc)
     * @see javax.servlet.Servlet#getServletConfig()
     */
    @Override
    public ServletConfig getServletConfig() {
        
        return this.delegate.getServletConfig();
    }

    /* (non-Javadoc)
     * @see javax.servlet.Servlet#getServletInfo()
     */
    @Override
    public String getServletInfo() {
        
        return this.delegate.getServletInfo();
    }

    /* (non-Javadoc)
     * @see javax.servlet.Servlet#init(javax.servlet.ServletConfig)
     */
    @Override
    public void init(ServletConfig config) throws ServletException {
        
        this.delegate.init(config);
    }

    /* (non-Javadoc)
     * @see javax.servlet.Servlet#service(javax.servlet.ServletRequest, javax.servlet.ServletResponse)
     */
    @Override
    public void service(ServletRequest req, ServletResponse resp)
            throws ServletException, IOException {
       
        try {
            
            if (log.isDebugEnabled()) {
                log.debug("Performing {}...",RequestHelper.describeRequest(req));
            }
            
            this.delegate.service(req,resp);

            if (log.isDebugEnabled()) {
                log.debug("{} finished successfully.",RequestHelper.describeRequest(req));
            }
        }
        catch (UnavailableException e) {
            
            if (e.isPermanent()) {
                
                log.error(RequestHelper.describeRequest(req) + " reported permanent unavailablability",e);

                throw new UnavailableException("permanently unavailable");
            }
            else {
                
                log.error(RequestHelper.describeRequest(req) +
                        " reported temporary unavailablability for ["+e.getUnavailableSeconds()+"] seconds",e);

                throw new UnavailableException("unavailable",e.getUnavailableSeconds());
            }
        }
        catch (ServletException e) {
            
            log.error(RequestHelper.describeRequest(req) + " reported a servlet exception",e);
            
            throw new ServletException();
        }
        catch (IOException e) {

            log.error(RequestHelper.describeRequest(req) + " reported an I/O exception",e);

            throw new IOException();
        }
        catch (Throwable e) {
            
            log.error(RequestHelper.describeRequest(req) + " reported an unspecific Throwable",e);
           
            throw new ServletException();
        }
    }
}
