/***********************************************************
 * $Id: HttpCheckLoginHelper.java 948 2014-03-22 17:01:57Z util $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 19.05.2011
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http.sec;

import java.io.IOException;
import java.security.Principal;
import java.util.Locale;
import java.util.TimeZone;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * A static helper for servlets, which perform access checks on behalf
 * of a {@link HttpLoginService}. 
 */
public abstract class HttpCheckLoginHelper {

	public static final String LOGIN_URL_HEADER = "X-HTTP-util-login-url";
	public static final String PAGE_TOKEN_HEADER = "X-HTTP-util-page-token";
    public static final String PAGE_TOKEN_VARIABLE = "pageToken";
    
	/**
	 * Call {@link HttpLoginService#checkLogin(HttpServletRequest)} and
	 * write a HTTP unauthorized status (401) with the login URL contained in
	 * the response header <code>{@value #LOGIN_URL_HEADER}</code>, if no user is logged in.
	 * 
	 * @param service The login service.
	 * @param req The servlet request.
	 * @param resp The servlet response.
	 * @return The logged in user. If <code>null</code> is returned,
	 *           a servlet response with status 401 has been written by this method
	 *           and the servlet has to return immediately.
	 * @throws IOException Upon errors writing a response.
	 * 
	 * @see #sendLoginRequired(HttpServletResponse, String)
	 */
	public static final Principal checkLogin(HttpLoginService service,
			HttpServletRequest req, HttpServletResponse resp) throws IOException {
		
		Principal user = service.checkLogin(req);
		
		if (user != null)
			return user;
		
		sendLoginRequired(resp,service.getLoginUrl());
		
		return null;
	}
	
	/**
	 * Try to extract a page token from a request by either
	 * the {@value #PAGE_TOKEN_HEADER} HTTP header or the
	 * value of the form parameter {@value #PAGE_TOKEN_VARIABLE},
	 * if the content type of the request is
	 * <code>application/x-www-form-urlencoded</code>.
	 * 
	 * @param req The request to check.
	 * @return The extracted counter-XSRF page token.
	 */
	public static final String getPageToken(HttpServletRequest req) {
	    
        String pageToken = req.getHeader(PAGE_TOKEN_HEADER);
        
        if (pageToken == null && "POST".equals(req.getMethod()) &&
                 "application/x-www-form-urlencoded".equals(req.getContentType())) {
            
            pageToken = req.getParameter(PAGE_TOKEN_VARIABLE);
        }

        return pageToken;
	}

    /**
     * Performs the following steps to authenticate an HTTP request:
     * 
     * <ol>
     * <li>Check the request for a {@value #PAGE_TOKEN_HEADER} header.</li>
     * <li>If no such header exists and the content type of the request
     *     is <code>application/x-www-form-urlencoded</code>,
     *     check the submitted form data for a parameter with
     *     name {@value #PAGE_TOKEN_VARIABLE}.</li>
     * <li>Check the so extracted page token using
     *     {@link PageTokenService#checkPageToken(HttpServletRequest, String)}</li>
     * <li>Call {@link HttpLoginService#checkLogin(HttpServletRequest)} to check
     *     for a user being logged in.
     * </ol>
     * If no user may be deduced by the above sequence,
     * write a HTTP unauthorized status (401) with the login URL contained in
     * the response header <code>{@value #LOGIN_URL_HEADER}</code>.
     * 
     * @param service The login service.
     * @param pageTokenService The XSRF page token service.
     * @param req The servlet request.
     * @param resp The servlet response.
     * @return The logged in user. If <code>null</code> is returned,
     *           a servlet response with status 401 has been written by this method
     *           and the servlet has to return immediately.
     * @throws IOException Upon errors writing a response.
     * 
     * @see #sendLoginRequired(HttpServletResponse, String)
     */
    public static final Principal checkLogin(HttpLoginService service,
            PageTokenService pageTokenService,
            HttpServletRequest req, HttpServletResponse resp) throws IOException {
        
        String pageToken = getPageToken(req);
        
        return checkLogin(service,pageTokenService,pageToken,req,resp);
    }

    /**
     * Performs the following steps to authenticate an HTTP request:
     * 
     * <ol>
     * <li>Check the caller-extracted page token using
     *     {@link PageTokenService#checkPageToken(HttpServletRequest, String)}</li>
     * <li>Call {@link HttpLoginService#checkLogin(HttpServletRequest)} to check
     *     for a user being logged in.
     * </ol>
     * If no user may be deduced by the above sequence,
     * write a HTTP unauthorized status (401) with the login URL contained in
     * the response header <code>{@value #LOGIN_URL_HEADER}</code>.
     * 
     * @param service The login service.
     * @param pageTokenService The XSRF page token service.
     * @param pageToken The page token deduced by the caller.
     * @param req The servlet request.
     * @param resp The servlet response.
     * @return The logged in user. If <code>null</code> is returned,
     *           a servlet response with status 401 has been written by this method
     *           and the servlet has to return immediately.
     * @throws IOException Upon errors writing a response.
     * 
     * @see #sendLoginRequired(HttpServletResponse, String)
     */
    public static final Principal checkLogin(HttpLoginService service,
            PageTokenService pageTokenService,
            String pageToken,
            HttpServletRequest req, HttpServletResponse resp) throws IOException {
        
        Principal user = null;        
        
        if (pageToken != null && pageTokenService.checkPageToken(req,pageToken)) {
            user = service.checkLogin(req);
        }
        
        if (user != null)
            return user;
        
        sendLoginRequired(resp,service.getLoginUrl());
        
        return null;
    }

    /**
	 * Return the login locale of the given request.
	 * This method takes extra care for Implementations of
	 * {@link HttpLoginService} compiled against http-util-1.5.0
	 * or earlier , where the method
	 * {@link HttpLoginService#getLocale(HttpServletRequest)} was
	 * not contained in the interface.
	 * 
	 * @param service The login service of choice.
	 * @param req The servlet request.
	 * @return The locale returned by
	 *         {@link HttpLoginService#getLocale(HttpServletRequest)}
	 *         or the locale returned by {@link HttpServletRequest#getLocale()},
	 *         if {@link HttpLoginService#getLocale(HttpServletRequest)} is not
	 *         implemented by the given service instance.
	 */
	public static final Locale getLoginLocale(HttpLoginService service, HttpServletRequest req) {

		Locale loc = null;
		
		try {
		    if (service != null) {
		        loc = service.getLocale(req);
		    }
		}
		catch (Throwable e) {
			// legacy http-util-1.5.0 or earlier HttpLoginService called,
			// where method getLocale() was missing and an AbstractError or
			// a proxy-initiated UnknownMethodError or InvocationTargetException
			// might be thrown.
		}
		
		if (loc == null) {
			loc = req.getLocale();
		}

		return loc;
	}
	
	/**
	 * Return the login time zone of the given request.
	 * This method takes extra care for Implementations of
	 * {@link HttpLoginService} compiled against http-util-1.6.0
	 * or earlier, where the method
	 * {@link HttpLoginService#getTimeZone(HttpServletRequest)} was
	 * not contained in the interface.
	 * 
	 * @param service The login service of choice.
	 * @param req The servlet request.
	 * @return The time zone returned by
	 *         {@link HttpLoginService#getTimeZone(HttpServletRequest)}
	 *         or the time zone returned by {@link TimeZone#getDefault()},
	 *         if {@link HttpLoginService#getTimeZone(HttpServletRequest)} is not
	 *         implemented by the given service instance.
	 */
	public static final TimeZone getLoginTimeZone(HttpLoginService service, HttpServletRequest req) {

		TimeZone tz = null;
		
		try {
		    if (service != null) {
		        tz = service.getTimeZone(req);
		    }
		}
		catch (Throwable e) {
			// legacy http-util-1.6.0 or earlier HttpLoginService called,
			// where method getTimeZone() was missing and an AbstractError or
			// a proxy-initiated UnknownMethodError or InvocationTargetException
			// might be thrown.
		}
		
		if (tz == null) {
			tz = TimeZone.getDefault();
		}

		return tz;
	}
	
	/**
	 * Write a HTTP unauthorized status (401) with the login URL contained in
	 * the response header <code>{@value #LOGIN_URL_HEADER}</code> as response
	 * to the client.
	 * 
	 * @param resp The servlet response.
	 * @param loginUrl A login URL as returned by {@link HttpLoginService#getLoginUrl()}.
	 * @throws IOException Upon errors writing to the servlet response.
	 */
	public static final void sendLoginRequired(HttpServletResponse resp, String loginUrl) throws IOException {
		
		resp.setHeader(LOGIN_URL_HEADER,loginUrl);
		resp.sendError(HttpServletResponse.SC_UNAUTHORIZED,"No user logged on.");
	}
	
	/**
	 * Call {@link HttpLoginService#checkPermission(HttpServletRequest, String)}
	 * on a context URL composed from the request URL without server and protocol.
	 * If permission is denied a HTTP forbidden response (403) is written
	 * to the response.
	 * 
	 * @param service The login service.
	 * @param req The servlet request.
	 * @param resp The servlet response.
	 * @return Whether permission is granted. If <code>false</code> is returned,
	 *           a servlet response has written and the servlet has
	 *           to return immediately.
	 * @throws IOException Upon errors writing a response.
	 */
	public static final boolean checkPermission(HttpLoginService service,
			HttpServletRequest req, HttpServletResponse resp) throws IOException {
		
		String query = req.getQueryString();
		
		String ctxt;
		
		if (query == null)
			ctxt = req.getRequestURI();
		else
			ctxt = req.getRequestURI() + "?" + query;
		
		if (service.checkPermission(req,ctxt))
			return true;
		
		resp.sendError(HttpServletResponse.SC_FORBIDDEN,"Access denied.");
		
		return false;
	}
}
